@@ -137,6 +137,9 @@ cairo_surface_t *regress_test_cairo_surface_full_return (void);
 void regress_test_cairo_surface_none_in (cairo_surface_t *surface);
 void regress_test_cairo_surface_full_out (cairo_surface_t **surface);
 
+/* versioning (deprecated, since, stability) */
+void regress_test_versioning ();
+
 GVariant *regress_test_gvariant_i (void);
 GVariant *regress_test_gvariant_s (void);
 GVariant *regress_test_gvariant_asv (void);
@@ -262,6 +265,7 @@ typedef struct _RegressTestStructA RegressTestStructA;
 typedef struct _RegressTestStructB RegressTestStructB;
 typedef struct _RegressTestStructC RegressTestStructC;
 typedef struct _RegressTestStructD RegressTestStructD;
+typedef struct _RegressTestStructF RegressTestStructF;
 
 struct _RegressTestStructA
 {
@@ -298,6 +302,7 @@ struct _RegressTestStructC
  * @field: (type RegressTestObj):
  * @list: (element-type RegressTestObj):
  * @garray: (element-type RegressTestObj):
+ * @ref_count:
  */
 struct _RegressTestStructD
 {
@@ -325,6 +330,18 @@ struct RegressTestStructE
   } some_union[2];
 };
 
+/* This one has members with const or volatile modifiers. */
+struct _RegressTestStructF
+{
+  volatile gint   ref_count;
+  const gint     *data1;
+  const gint     *const data2;
+  const gint     *const *const data3;
+  const gint    **const* data4;
+  volatile gint  *const data5;
+  const gint     *volatile data6;
+};
+
 /* plain-old-data boxed types */
 typedef struct _RegressTestSimpleBoxedA RegressTestSimpleBoxedA;
 typedef struct _RegressTestSimpleBoxedB RegressTestSimpleBoxedB;
@@ -861,4 +878,15 @@ void regress_test_struct_fixed_array_frob (RegressTestStructFixedArray *str);
 void regress_has_parameter_named_attrs (int        foo,
                                         gpointer   attributes);
 
+
+typedef struct {
+  int dummy;
+  struct {
+    const char *name;
+    guint x;
+  } attributes[32];
+
+  double dummy2;
+} RegressLikeGnomeKeyringPasswordSchema;
+
 #endif /* __GITESTTYPES_H__ */
